/**
 * @file tick_minicom.c
 * @brief a tiny uart client
 * @license GPL (General Public License)
 */ 
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include "debug.h"
#include "util.h"

#define DEFAULT_UART "/dev/ttySAC0"
#define DEFAULT_BAUD 115200

static int keep_run=1;



/// Set '&' as the escape code
void input_command(int input,int output)  {
    int i;
    char buf[1];
    while(keep_run)  {
        i=getche(input);
        if (i< 0) {
            debug("file closed!!\n");
            keep_run=0;
            break;
        }
        if (i==10)  {  // LF
            i='\r';
        }
        if (i=='&') {
            debug("User entered the escape code.\n");
            debug("Exiting program!\n");
            keep_run=0;
            break;
        }
        buf[0]=i;
        write(output,buf,1);
    }   
}

void read_from_uart(int input,int output) {
    int i;
    char buf[1];
    while (keep_run) {
        i=getche(input);
        buf[0]=i;
        i=write(output,buf,1);
    }
}

int main (int args,char **argv) {
    int uart;
    int pid;
    char buf[128];
    if (args==1) {
        uart=open_device(DEFAULT_UART,DEFAULT_BAUD);
    } else if (args==2) {
        uart=open_device(argv[1],DEFAULT_BAUD);
    } else {
        uart=open_device(argv[1],atoi(argv[2]));
    }
    if (uart<0) { 
        error("Open device failed !!\n");
        return 0;
    }else {
        debug("Open Uart and set baud rate OK!!\n");
    }
    keep_run=1;
    pid=fork();
    if (pid) {  // parent
        input_command(0,uart);    
        debug("Exiting Tick's minicom\n");
    }else {  // child
        pid = getppid();
        read_from_uart(uart,1);    
        debug("Try to close uart\n");
        debug("Result of close uart=%d\n",close(uart));
    }
    sprintf(buf,"kill -9 %d",pid);
    system(buf);
}
