#include "util.h"
#include "debug.h"
#include <string.h>

int getche (int fd) {
	int i;
	char ch;
	struct termios old, new;
	tcgetattr (fd, &old);
	memcpy (&new,&old, sizeof (struct termios));
	new.c_lflag = ~(ICANON | ECHO);
	tcsetattr (fd, TCSANOW,&new);
	i = read (fd,&ch, 1);
	tcsetattr (fd, TCSANOW, &old);
	if (i == 1) {
		return ch;
	}
	else {
            debug("fd %d ERROR\n",fd);
            return -1;
	}
}

int set_baud(int fd,int rate) {
    int i,rt;
    u_int32_t bd = 0;
    struct termios ti;
    for (i = 0; i < ARRAY_SIZE(bdrts); i++) {
        if (bdrts[i].bps == rate) {
            bd = bdrts[i].b;
            break;
        }
    }
    if (bd==0) {    
        error("Cannot set Baud Rate!!\n");
        return -1;
    }
    rt = tcgetattr(fd, &ti);
    if (rt<0) {
        error ("Cannot get attr i=%d\n",rt);
        return rt;
    }
    rt = cfsetispeed(&ti, B0);
    if (rt<0) {  
        error("Cannot set baud to B0 for initial!!\n");
        return rt;
    }
    debug("Open uart with speed %d\n",bdrts[i].bps);
    rt = cfsetospeed(&ti, bd);
    if (rt < 0) {
        error("Cannot set baud rate %d\n",bdrts[i].bps);
        return rt;
    }
    //ti.c_cflag &= ~CRTSCTS;  // NO hardware control
    ti.c_cflag |= CRTSCTS;  // hardware control
    return tcsetattr(fd, 0, &ti);
}

/**
 * Open Uart for testing 
 * return value is the fd
 */ 
int  open_device (char *device,int speed) {
    int fd;
    if (device==NULL) {
        error("device is invalid (NULL pointer)\n");
        return -1;
    }
    debug("Try to open device %s!\n",device);
    fd = open(device,O_RDWR);
    if (fd < 0) {
        error("cannot open uart %s, fd is %d\n",device,fd);
        error("%s\n",strerror(errno));
        return -1;
    }
    if (set_baud(fd,speed) < 0)  {
        error("Set Baud rate failed!!\n");
        return -1;
    }
    return fd;
}
